@@ -484,7 +484,7 @@ get_cpu_info ()
   /* count duplicates */
   for (i = 0; i != info->ncpu; ++i)
     {
-      const char * const keys[] = { "model name", "cpu", "Processor" };
+      const char * const keys[] = { "model name", "cpu", "Processor", "Model Name" };
       char *model;
       int  *count;
 
@@ -930,8 +930,6 @@ cc_system_details_window_class_init (CcSystemDetailsWindowClass *klass)
 {
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-  gtk_widget_class_add_binding_action (widget_class, GDK_KEY_Escape, 0, "window.close", NULL);
-  
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/info-overview/cc-system-details-window.ui");
 
   gtk_widget_class_bind_template_child (widget_class, CcSystemDetailsWindow, toast_overlay);
