@@ -463,6 +463,18 @@ shortcut_search_entry_changed_cb (CcKeyboardShortcutDialog *self)
   shortcut_search_result_changed_cb (self);
 }
 
+static void
+shortcut_search_entry_stopped_cb (CcKeyboardShortcutDialog *self)
+{
+  const char *search_text;
+  search_text = gtk_editable_get_text (GTK_EDITABLE (self->search_entry));
+
+  if (search_text && g_strcmp0 (search_text, "") != 0)
+    gtk_editable_set_text (GTK_EDITABLE (self->search_entry), "");
+  else
+    gtk_window_close (GTK_WINDOW (self));
+}
+
 static void
 shortcut_section_row_activated_cb (CcKeyboardShortcutDialog *self,
                                    GtkListBoxRow            *row)
@@ -543,6 +555,7 @@ cc_keyboard_shortcut_dialog_class_init (CcKeyboardShortcutDialogClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, reset_all_clicked_cb);
   gtk_widget_class_bind_template_callback (widget_class, shortcut_dialog_visible_page_changed_cb);
   gtk_widget_class_bind_template_callback (widget_class, shortcut_search_entry_changed_cb);
+  gtk_widget_class_bind_template_callback (widget_class, shortcut_search_entry_stopped_cb);
   gtk_widget_class_bind_template_callback (widget_class, shortcut_section_row_activated_cb);
 }
 
